#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_portal.interfaces import IPortalPage, IPortalContext, IPortalTemplateConfiguration, \
    IPortalTemplate, PORTAL_PAGE_KEY, IPortalPortletsConfiguration, PORTLETS_CONFIGURATION_KEY, \
    ILocalTemplateHandler
from zope.annotation.interfaces import IAnnotations
from zope.traversing.interfaces import ITraversable

# import packages
from persistent import Persistent
from pyams_portal.template import PortalTemplate
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.registry import query_utility
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.copy import clone
from zope.interface import implementer, alsoProvides, noLongerProvides
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty


@implementer(IPortalPage)
class PortalPage(Persistent, Contained):
    """Portal page persistent class"""

    _inherit_parent = FieldProperty(IPortalPage['inherit_parent'])
    _use_local_template = FieldProperty(IPortalPage['use_local_template'])
    _shared_template = FieldProperty(IPortalPage['shared_template'])
    _local_template = FieldProperty(IPortalPage['local_template'])

    @property
    def can_inherit(self):
        return IPortalContext.providedBy(self.__parent__.__parent__)

    @property
    def inherit_parent(self):
        return self._inherit_parent if self.can_inherit else False

    @inherit_parent.setter
    def inherit_parent(self, value):
        if (not value) or self.can_inherit:
            self._inherit_parent = value

    @property
    def parent(self):
        parent = self.__parent__
        page = IPortalPage(parent)
        while page.inherit_parent:
            parent = parent.__parent__
            page = IPortalPage(parent)
        return parent

    @property
    def use_local_template(self):
        return self._use_local_template if not self.inherit_parent else False

    @use_local_template.setter
    def use_local_template(self, value):
        self._use_local_template = value
        if value and (self._local_template is None) and not self.inherit_parent:
            registry = get_current_registry()
            template = PortalTemplate()
            template.name = "local"
            self._local_template = template
            registry.notify(ObjectCreatedEvent(template))
            locate(template, self, '++template++')
        if self.use_local_template:
            alsoProvides(self, ILocalTemplateHandler)
        else:
            noLongerProvides(self, ILocalTemplateHandler)

    @property
    def shared_template(self):
        return IPortalPage(self.parent).shared_template if self.inherit_parent else self._shared_template

    @shared_template.setter
    def shared_template(self, value):
        if not self.inherit_parent:
            if IPortalTemplate.providedBy(value):
                value = value.name
            self._shared_template = value

    @property
    def local_template(self):
        return IPortalPage(self.parent).local_template if self.inherit_parent else self._local_template

    @local_template.setter
    def local_template(self, value):
        if not self.inherit_parent:
            self._local_template = value

    @property
    def template(self):
        if self.use_local_template:
            template = self.local_template
        else:
            template = self.shared_template
        if isinstance(template, str):
            template = query_utility(IPortalTemplate, name=template)
        return template


@adapter_config(context=IPortalContext, provides=IPortalPage)
def PortalContextPageAdapter(context):
    """Portal context page factory"""
    annotations = IAnnotations(context)
    page = annotations.get(PORTAL_PAGE_KEY)
    if page is None:
        page = annotations[PORTAL_PAGE_KEY] = PortalPage()
        get_current_registry().notify(ObjectCreatedEvent(page))
        locate(page, context)
    return page


@adapter_config(name='template', context=IPortalContext, provides=ITraversable)
class PortalContextTemplateTraverser(ContextAdapter):
    """++template++ portal context traverser"""

    def traverse(self, name, furtherpath=None):
        page = IPortalPage(self.context)
        if page.use_local_template:
            return page.local_template


@adapter_config(context=IPortalContext, provides=IPortalTemplateConfiguration)
def PortalContextTemplateConfigurationAdapter(context):
    """Portal context template configuration adapter"""
    template = IPortalPage(context).template
    return IPortalTemplateConfiguration(template)


@adapter_config(context=IPortalContext, provides=IPortalPortletsConfiguration)
def PortalContextPortletsConfigurationAdapter(context):
    """Portal context portlets configuration adapter"""
    page = IPortalPage(context)
    if page.inherit_parent:
        return IPortalPortletsConfiguration(page.parent)
    else:
        annotations = IAnnotations(context)
        config = annotations.get(PORTLETS_CONFIGURATION_KEY)
        if config is None:
            config = annotations[PORTLETS_CONFIGURATION_KEY] = clone(IPortalPortletsConfiguration(page.template))
            get_current_registry().notify(ObjectCreatedEvent(config))
            locate(config, context)
            for portlet_id, portlet_config in config.items():
                portlet_cloned_config = clone(portlet_config)
                config.set_portlet_configuration(portlet_id, portlet_cloned_config)
        return config


@adapter_config(name='portlet', context=IPortalContext, provides=ITraversable)
class PortalContextPortletTraverser(ContextAdapter):
    """++portlet++ portal context traverser"""

    def traverse(self, name, thurtherpath=None):
        config = IPortalPortletsConfiguration(self.context)
        if name:
            return config.get_portlet_configuration(int(name))
        else:
            return config
