#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import logging
logger = logging.getLogger('PyAMS (portal)')

import venusian

# import interfaces
from pyams_portal.interfaces import IPortlet, IPortletSettings, IPortletConfiguration, IPortletPreviewer, \
    IPortletRenderer, IPortalPortletsConfiguration, IPortalTemplate, IPortalContext, IPortalPage
from zope.traversing.interfaces import ITraversable

# import packages
from persistent import Persistent
from persistent.mapping import PersistentMapping
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.request import check_request
from pyams_utils.vocabulary import vocabulary_config
from pyams_viewlet.viewlet import ContentProvider
from pyramid.exceptions import ConfigurationError
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.copy import clone
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


#
# Portlets utilities
#

@implementer(IPortlet)
class Portlet(object):
    """Base portlet utility"""

    permission = FieldProperty(IPortlet['permission'])

    toolbar_image = None
    toolbar_css_class = 'fa fa-fw fa-2x fa-edit'

    settings_class = None


class portlet_config(object):
    """Class decorator used to declare a portlet"""

    venusian = venusian  # for testing injection

    def __init__(self, **settings):
        self.__dict__.update(settings)

    def __call__(self, wrapped):
        settings = self.__dict__.copy()
        depth = settings.pop('_depth', 0)

        def callback(context, name, ob):
            name = settings.get('name') or getattr(ob, 'name', None)
            if name is None:
                raise ConfigurationError("You must provide a name for a portlet")

            permission = settings.get('permission')
            if permission is not None:
                ob.permission = permission

            if type(ob) is type:
                factory = ob
                component = None
            else:
                factory = None
                component = ob

            config = context.config.with_package(info.module)
            logger.debug("Registering portlet {0} named '{1}'".format(str(component) if component else str(factory),
                                                                      name))
            config.registry.registerUtility(component=component, factory=factory,
                                            provided=IPortlet, name=name)

        info = self.venusian.attach(wrapped, callback, category='pyams_portal',
                                    depth=depth + 1)
        if info.scope == 'class':
            # if the decorator was attached to a method in a class, or
            # otherwise executed at class scope, we need to set an
            # 'attr' into the settings if one isn't already in there
            if settings.get('attr') is None:
                settings['attr'] = wrapped.__name__

        settings['_info'] = info.codeinfo  # fbo "action_method"
        return wrapped


@vocabulary_config(name='PyAMS portal portlets')
class PortletVocabulary(SimpleVocabulary):
    """Portlet vocabulary"""

    def __init__(self, context):
        request = check_request()
        translate = request.localizer.translate
        utils = request.registry.getUtilitiesFor(IPortlet)
        terms = [SimpleTerm(name, title=translate(util.label))
                 for name, util in sorted(utils, key=lambda x: translate(x[1].label))]
        super(PortletVocabulary, self).__init__(terms)


#
# Portlets renderers
#

class PortletContentProvider(ContentProvider):
    """Base portlet content provider"""

    def __init__(self, context, request, view, settings):
        super(PortletContentProvider, self).__init__(context, request, view)
        self.__parent__ = view
        self.settings = settings
        self.portlet = self.request.registry.getUtility(IPortlet, name=settings.configuration.portlet_name)

    def __call__(self):
        if not self.settings.visible:
            return ''
        if self.portlet.permission and not self.request.has_permission(self.portlet.permission):
            return ''
        self.update()
        return self.render()


@implementer(IPortletPreviewer)
class PortletPreviewer(PortletContentProvider):
    """Portlet previewer adapter"""


@implementer(IPortletRenderer)
class PortletRenderer(PortletContentProvider):
    """Portlet renderer adapter"""


#
# Portlet configuration
#

@implementer(IPortletSettings)
class PortletSettings(Persistent, Contained):
    """Portlet settings persistent class"""

    visible = FieldProperty(IPortletSettings['visible'])

    __name__ = '++settings++'

    def __init__(self, configuration):
        self.__parent__ = configuration

    @property
    def configuration(self):
        return self.__parent__


@implementer(IPortletConfiguration)
class PortletConfiguration(Persistent, Contained):
    """Portlet configuration persistent class

    PortletConfiguration.__parent__ points to context where configuration is applied
    PortletConfiguration.parent points to context from where configuration is inherited
    """

    portlet_id = FieldProperty(IPortletConfiguration['portlet_id'])
    portlet_name = None
    _inherit_parent = FieldProperty(IPortletConfiguration['inherit_parent'])
    _settings = FieldProperty(IPortletConfiguration['settings'])

    def __init__(self, portlet):
        self.portlet_name = portlet.name
        self._settings = portlet.settings_class(self)

    @property
    def can_inherit(self):
        return not IPortalTemplate.providedBy(self.__parent__)

    @property
    def inherit_parent(self):
        return self._inherit_parent if self.can_inherit else False

    @inherit_parent.setter
    def inherit_parent(self, value):
        if (not value) or self.can_inherit:
            self._inherit_parent = value

    @property
    def parent(self):
        parent = self.__parent__
        if IPortalTemplate.providedBy(parent):
            return parent
        while IPortalContext.providedBy(parent):
            configuration = IPortalPortletsConfiguration(parent).get_portlet_configuration(self.portlet_id)
            if not configuration.inherit_parent:
                return parent
            if not IPortalContext.providedBy(parent.__parent__):
                break
            parent = parent.__parent__
        page = IPortalPage(parent, None)
        if page is not None:
            return page.template

    @property
    def settings(self):
        if self.inherit_parent:
            return IPortalPortletsConfiguration(self.parent).get_portlet_configuration(self.portlet_id).settings
        else:
            return self._settings

    @property
    def editor_settings(self):
        return self._settings


@adapter_config(context=IPortlet, provides=IPortletConfiguration)
def PortletConfigurationAdapter(portlet):
    """Portlet configuration factory"""
    return PortletConfiguration(portlet)


@adapter_config(context=IPortletConfiguration, provides=IPortletSettings)
def PortletConfigurationSettingsAdapter(configuration):
    """Portlet configuration settings adapter"""
    return configuration.settings


@adapter_config(name='settings', context=IPortletConfiguration, provides=ITraversable)
class PortletConfigurationSettingsTraverser(ContextAdapter):
    """++settings++ portlet configuration traverser"""

    def traverse(self, name, furtherpath=None):
        return self.context.settings


#
# Template portlets configuration
#

@implementer(IPortalPortletsConfiguration)
class PortalPortletsConfiguration(PersistentMapping, Contained):
    """Portal portlets configuration"""

    def get_portlet_configuration(self, portlet_id):
        configuration = self.get(portlet_id)
        if (configuration is None) and not IPortalTemplate.providedBy(self.__parent__):
            template = IPortalPage(self.__parent__).template
            portlets = IPortalPortletsConfiguration(template)
            configuration = clone(portlets.get_portlet_configuration(portlet_id))
            get_current_registry().notify(ObjectCreatedEvent(configuration))
            self.set_portlet_configuration(portlet_id, configuration)
        return configuration

    def set_portlet_configuration(self, portlet_id, config):
        config.portlet_id = portlet_id
        self[portlet_id] = config
        locate(config, self.__parent__, '++portlet++{0}'.format(portlet_id))

    def delete_portlet_configuration(self, portlet_id):
        if isinstance(portlet_id, int):
            portlet_id = (portlet_id,)
        for p_id in portlet_id:
            del self[p_id]
