#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_portal.interfaces import IPortalTemplateContainer, IPortalTemplate, IPortalTemplateContainerConfiguration, \
    MANAGE_TEMPLATE_PERMISSION
from pyams_portal.zmi.interfaces import IPortalTemplateContainerMenu
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.interfaces.container import ITable, ITableElementEditor
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION, MANAGE_SYSTEM_PERMISSION
from pyams_zmi.interfaces.menu import IControlPanelMenu
from pyams_zmi.layer import IAdminLayer
from z3c.table.interfaces import IColumn, IValues
from zope.component.interfaces import ISite

# import packages
from pyams_form.form import AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.container import ContainerView
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import DefaultElementEditorAdapter, BaseTable, NameColumn, TrashColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogEditForm
from pyams_zmi.view import AdminView
from pyramid.url import resource_url
from pyramid.view import view_config
from z3c.form import field
from zope.interface import implementer, Interface

from pyams_portal import _


#
# Template container views
#

@adapter_config(context=(IPortalTemplateContainer, IAdminLayer, ITable), provides=ITableElementEditor)
class PortalTemplateContainerTableElementEditor(DefaultElementEditorAdapter):
    """Portal template container table element editor"""

    view_name = 'portal-templates.html'
    modal_target = False

    @property
    def url(self):
        site = get_parent(self.context, ISite)
        return resource_url(site, self.request, 'admin#{0}'.format(self.view_name))


@viewlet_config(name='portal-templates.menu', context=ISite, layer=IAdminLayer, manager=IControlPanelMenu,
                permission=VIEW_SYSTEM_PERMISSION, weight=20)
@viewletmanager_config(name='portal-templates.menu', context=ISite, layer=IAdminLayer)
@implementer(IPortalTemplateContainerMenu)
class PortalTemplateContainerMenuItem(MenuItem):
    """Portal template container menu"""

    label = _("Portal templates")
    icon_class = 'fa-columns'
    url = '#portal-templates.html'


class PortalTemplateContainerTable(BaseTable):
    """Portal template container table"""

    id = 'portal_templates_table'
    title = _("Shared portal templates")

    @property
    def data_attributes(self):
        manager = query_utility(IPortalTemplateContainer)
        attributes = super(PortalTemplateContainerTable, self).data_attributes
        table_attrs = {'data-ams-location': absolute_url(manager, self.request),
                       'data-ams-plugins': 'pyams_workflow pyams_portal',
                       'data-ams-plugin-pyams_workflow-src': "/--static--/pyams_workflow/js/workflow{MyAMS.devext}.js",
                       'data-ams-plugin-pyams_portal-src': "/--static--/pyams_portal/js/portal{MyAMS.devext}.js",
                       'data-ams-plugin-pyams_portal-css': "/--static--/pyams_portal/css/portal{MyAMS.devext}.css"}
        if 'table' in attributes:
            attributes['table'].update(table_attrs)
        else:
            attributes['table'] = table_attrs
        return attributes


@adapter_config(context=(IPortalTemplate, IAdminLayer, PortalTemplateContainerTable), provides=ITableElementEditor)
class PortalTemplateTableElementEditor(DefaultElementEditorAdapter):
    """Portal template table element editor"""

    modal_target = False

    @property
    def url(self):
        return resource_url(self.context, self.request, 'admin#{0}'.format(self.view_name))


@adapter_config(name='name', context=(Interface, IAdminLayer, PortalTemplateContainerTable), provides=IColumn)
class PortalTemplateContainerNameColumn(NameColumn):
    """Portal template container name column"""

    attrName = 'name'


@adapter_config(name='trash', context=(Interface, IAdminLayer, PortalTemplateContainerTable), provides=IColumn)
class PortalTemplateContainerTrashColumn(TrashColumn):
    """Portal template container trash column"""

    icon_hint = _("Delete template")
    permission = MANAGE_TEMPLATE_PERMISSION


@adapter_config(context=(ISite, IAdminLayer, PortalTemplateContainerTable), provides=IValues)
class PortalTemplateContainerValuesAdapter(ContextRequestViewAdapter):
    """Portal template container values adapter"""

    @property
    def values(self):
        manager = query_utility(IPortalTemplateContainer)
        if manager is not None:
            return manager.values()
        return ()


@pagelet_config(name='portal-templates.html', context=ISite, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@implementer(IInnerPage)
class PortalTemplateContainerView(AdminView, ContainerView):
    """Portal template container view"""

    table_class = PortalTemplateContainerTable

    def __init__(self, context, request):
        super(PortalTemplateContainerView, self).__init__(context, request)


@adapter_config(context=(ISite, IAdminLayer, PortalTemplateContainerView), provides=IPageHeader)
class PortalTemplateContainerHeaderAdapter(DefaultPageHeaderAdapter):
    """Portal template container header adapter"""

    icon_class = 'fa fa-fw fa-columns'


#
# Templates container configuration views
#

@viewlet_config(name='templates-container-configuration.menu', context=ISite, layer=IAdminLayer,
                manager=IPortalTemplateContainerMenu, permission=VIEW_SYSTEM_PERMISSION, weight=1)
class PortalTemplatesContainerPropertiesMenu(MenuItem):
    """Portal template container configuration menu"""

    label = _("Selected portlets...")
    icon_class = 'fa-thumb-tack'

    url = 'properties.html'
    modal_target = True

    def get_url(self):
        container = query_utility(IPortalTemplateContainer)
        return absolute_url(container, self.request, self.url)


@pagelet_config(name='properties.html', context=IPortalTemplateContainer, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class PortalTemplateContainerPropertiesEditForm(AdminDialogEditForm):
    """Portal template container properties edit form"""

    title = _("Portal templates container")
    legend = _("Edit selected portlets")
    icon_css_class = 'fa fa-fw fa-thumb-tack'

    fields = field.Fields(IPortalTemplateContainerConfiguration)
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    label_css_class = 'control-label col-md-4'
    input_css_class = 'col-md-8'


@view_config(name='properties.json', context=IPortalTemplateContainer, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class PortalTemplateContainerPropertiesAJAXEditForm(AJAXEditForm, PortalTemplateContainerPropertiesEditForm):
    """Portal template container properties edit form, JSON renderer"""
